﻿// Copyright (C) 2003, 2005 Daisuke Arai <darai@users.sourceforge.jp>
// Copyright (C) 2008 panacoran <panacoran@users.sourceforge.jp>
// 
// This program is part of Protra.
//
// Protra is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
// 
// $Id: LiteralNode.cs 306 2010-03-19 13:21:22Z panacoran $

using System;

namespace Protra.Lib.Lang
{
	/// <summary>
	/// リテラルのノードを表すクラス。
	/// </summary>
	public class LiteralNode : Node
	{
		/// <summary>
		/// 値
		/// </summary>
		private Value val;

		/// <summary>
		/// コンストラクタ
		/// </summary>
		public LiteralNode() {}

		/// <summary>
		/// コンストラクタ
		/// </summary>
		/// <param name="val">値</param>
		public LiteralNode(Value val)
		{
			this.val = val;
		}

		/// <summary>
		/// 構文解析する。
		/// </summary>
		/// <exception cref="Protra.Lib.Lang.ParseException">
		/// 構文解析中にエラーが発生したときにthrowされる。
		/// </exception>
        /// <return>
        /// 解析した結果のノード。
        /// </return>
		public override Node Parse()
		{
			Token = Scanner.Token;
            switch (Token.Type)
            {
            case TokenType.Int:
                val = new Value(Token.IntValue);
                break;
            case TokenType.Float:
                val = new Value(Token.FloatValue);
                break;
            case TokenType.String:
                val = new Value(Token.Value);
                break;
            }
			Scanner.Scan();
            return this;
		}

		/// <summary>
		/// プログラムを実行する。
		/// </summary>
		/// <exception cref="Protra.Lib.Lang.RuntimeException">
		/// プログラム実行中にエラーが発生した場合にthrowされる。
		/// </exception>
		/// <param name="resource">リソース</param>
		/// <param name="at">@作用素の値</param>
		/// <returns>実行結果</returns>
		public override Value Execute(Resource resource, int at)
		{
			return val;
		}
	}
}
