// Copyright (C) 2003, 2005 Daisuke Arai <darai@users.sourceforge.jp>
// 
// This program is part of Protra.
//
// Protra is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
// 
// $Id: ComparisonNode.cs,v 1.4 2007-12-27 08:34:39 panacoran Exp $

using System;
using System.Collections;

namespace Protra.Lib.Lang
{
	/// <summary>
	/// 召r̃m[h\NXłB
	/// </summary>
	public class ComparisonNode : Node
	{
		/// <summary>
		/// m[h̃Xg
		/// </summary>
		private ArrayList nodeList = new ArrayList();
		/// <summary>
		/// Z̎ނ\̃Xg
		/// </summary>
		private ArrayList opStrList = new ArrayList();

		/// <summary>
		/// ͂܂B
		/// </summary>
		/// <exception cref="Protra.Lib.Lang.ParseException">
		/// ɃG[ꍇthrow܂B
		/// </exception>
		/// <param name="context">ReLXg</param>
		public override void Parse(Context context)
		{
			this.ContextToken = (ContextToken)context.Current;

			Node node = new ArithmeticExpressionNode();
			node.Parse(context);
			nodeList.Add(node);
			ContextToken ct = (ContextToken)context.Current;
			string token = ct.ToString();
			while(token == "<" || token == ">"
				|| token == "<=" || token == ">=")
			{
				context.MoveNext();
				node = new ArithmeticExpressionNode();
				node.Parse(context);
				nodeList.Add(node);
				opStrList.Add(token);
				ct = (ContextToken)context.Current;
				token = ct.ToString();
			}
		}

		/// <summary>
		/// \؂ȗ܂B
		/// </summary>
		/// <returns>ȗꂽm[h</returns>
		public override Node Simplify()
		{
			if(nodeList.Count == 1)
				return ((Node)nodeList[0]).Simplify();
			else
			{
				for(int i = 0; i < nodeList.Count; i++)
				{
					Node node = (Node)nodeList[i];
					nodeList[i] = node.Simplify();
				}
				return this;
			}
		}

		/// <summary>
		/// vOs܂B
		/// </summary>
		/// <exception cref="Protra.Lib.Lang.RuntimeException">
		/// vOsɃG[ꍇthrow܂B
		/// </exception>
		/// <param name="resource">\[X</param>
		/// <param name="at">@pf̒l</param>
		/// <returns>s</returns>
		public override Value Execute(Resource resource, int at)
		{
			try
			{
				Node node = (Node)nodeList[0];
				Value val1 = node.Execute(resource, at);
				for(int i = 1; i < nodeList.Count; i++)
				{
					node = (Node)nodeList[i];
					Value val2 = node.Execute(resource, at);

					string op = (string)opStrList[i - 1];
					switch(op)
					{
						case "<":
							val1 = new Value(val1 < val2);
							break;
						case ">":
							val1 = new Value(val1 > val2);
							break;
						case "<=":
							val1 = new Value(val1 <= val2);
							break;
						default:
							val1 = new Value(val1 >= val2);
							break;
					}
				}

				return val1;
			}
			catch(RuntimeException exc)
			{
				if (exc.ContextToken == null)
					exc = new RuntimeException(exc.Message, this.ContextToken);
				throw exc;
			}		
		}
	}
}
